from django.contrib.auth.models import AnonymousUser
from django.http.response import JsonResponse
from django.utils.deprecation import MiddlewareMixin
from rest_framework.authtoken.models import Token
from rest_framework import status
from .models import APIPermissionModel
from .exceptions import APIPermissionException
import logging
import re


logger = logging.getLogger(__name__)


class APIPermCheckMiddleware(MiddlewareMixin):

    def process_request(self, request):
        path = request.path
        method = request.method
        header_token = request.META.get('HTTP_AUTHORIZATION', None)
        user = request.user or AnonymousUser()
        if request.user and header_token is not None:
            try:
                token = header_token.split(' ')
                token_obj = Token.objects.get(key=token[1])
                user = token_obj.user
            except Token.DoesNotExist as e:
                logger.warning(f"api_permission checker: bearer token invalid: {e}")
                raise APIPermissionException("Bearer Token invalid")
            except Exception as e:
                logger.warning(f"APIPermissionException : {e}")
                raise APIPermissionException(e)

        logger.debug(f"header_token is:{header_token} user: {user}, method: {method}, path: {path}")
        if not path.startswith('/admin/'):
            if not self._has_permission(path, user, method):
                res = JsonResponse({
                    'code': 1,
                    'msg': f'permission denied: user: {user}, method: {method}, path: {path}',
                }, status=status.HTTP_403_FORBIDDEN)
                return res

    def _has_permission(self, path, user, method):
        groups = user.groups.all()
        queryset = APIPermissionModel.objects.filter(group__in=groups, method__in=[method, APIPermissionModel.ALL], active=True)
        logger.debug(f'api permission queryset count: {queryset.count()}')
        for api in queryset:
            if re.match(api.pattern, path):
                if api.method in [APIPermissionModel.ALL, method]:
                    return True
                else:
                    logger.debug(f"permission denied: user: {user} api.method:{api.method}, method: {method}")
            else:
                logger.info(f"path not match: user: {user} api.pattern:{api.pattern}, path: {path}")
        return False



from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='APIPermissionModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('pattern', models.CharField(max_length=128, verbose_name='API正则')),
                ('method', models.CharField(choices=[('POST', '创建/编辑/删除'), ('GET', '查看'), ('ALL', '所有')], max_length=16, verbose_name='方法类型')),
                ('active', models.BooleanField(default=True, verbose_name='是否生效')),
                ('comment', models.CharField(blank=True, max_length=256, null=True, verbose_name='备注')),
                ('created_at', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('updated_at', models.DateTimeField(auto_now=True, null=True, verbose_name='更新时间')),
                ('group', models.ManyToManyField(related_name='api_permissions', to='auth.Group', verbose_name='组/角色')),
            ],
            options={
                'verbose_name': 'API 权限',
                'verbose_name_plural': 'API 权限列表',
                'db_table': 'api_permission',
            },
        ),
    ]



from django.contrib import admin
from .models import APIPermissionModel


@admin.register(APIPermissionModel)
class ApiPermissionAdmin(admin.ModelAdmin):
    readonly_fields = ['created_at', 'updated_at']
    list_display = ['pattern', 'method', 'active', 'created_at']



from django.db import models
from django.contrib.auth.models import Group


class APIPermissionModel(models.Model):
    POST = 'POST'
    GET = 'GET'
    ALL = 'ALL'
    METHODS = (
        (POST, "创建/编辑/删除"),
        (GET, "查看"),
        (ALL, "所有"),
    )

    pattern = models.CharField("API正则", max_length=128)
    method = models.CharField("方法类型", max_length=16, choices=METHODS)
    active = models.BooleanField("是否生效", default=True)
    group = models.ManyToManyField(Group, db_constraint=True, verbose_name="组/角色",
                                   related_name='api_permissions')
    comment = models.CharField("备注", max_length=256, blank=True, null=True)

    created_at = models.DateTimeField("创建时间", auto_now_add=True, null=True)
    updated_at = models.DateTimeField("更新时间", auto_now=True, null=True)

    class Meta:
        db_table = 'api_permission'
        verbose_name = 'API 权限'
        verbose_name_plural = 'API 权限列表'
